/**
 * GebuehrenErhoehungsTatbestand.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.gebuehren;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Eine Klasse, um Mehrvertretungsgebühren abzubilden, also Gebühren, die die
 * Verfahrensgebühr um einen bestimmten Satz erhöhen. Und zwar umso mehr, je
 * mehr Leute (anzahl) vertreten werden, aber maximal bis zu einer bestimmten
 * Höhe.
 *
 * @author Peter Schuster (setrok)
 * @date 25.08.2014
 *
 */
public class GebuehrenErhoehungsTatbestand extends GebuehrenSatzTatbestand {

	private static final long serialVersionUID = -2255864991661667203L;
	/**
	 * jeweilsSatz speichert, wie hoch der Gebührensatz je zusätzlicher Einheit
	 * steigt, als double
	 */
	private/* final */double jeweilsSatz;
	/**
	 * maxSatz speichert, wie hoch der Satz höchstens sein darf, als double
	 */
	private/* final */double maxSatz;
	/**
	 * anzahl speichert die Anzahl der Einheiten, z.B. der zusätzlich
	 * Vertretenen, als int
	 */
	private int anzahl;

	/**
	 * Der Standardkonstruktor erlaubt, alle Felder bei der Initialisierung
	 * schon zu setzen.
	 * 
	 * @param bezeichnung
	 *            Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *            Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param gebuehrenKlasse
	 *            Klasse der Gebühr, also den Verweis auf die Klasse eines
	 *            Ablegers von GebuehrenTabelle, also entweder
	 *            GerichtsGebuehrenTabelle.class oder
	 *            AnwaltsGebuehrenTabelle.class
	 * @param jeweilsSatz
	 *            der Satz, der für jeden der in {@link #anzahl} genannten
	 *            Personen jeweils gilt
	 * @param maxSatz
	 *            der Satz, der maximal anzusetzen ist
	 * @param anzahl
	 *            die Anzahl der Personen (oder sonstigen Einheiten).
	 */
	public GebuehrenErhoehungsTatbestand(String bezeichnung,
			Class<? extends GebuehrenTabelle> gebuehrenKlasse,
			double jeweilsSatz, double maxSatz, int anzahl) {
		super(bezeichnung, -1.0, gebuehrenKlasse);
		this.jeweilsSatz = jeweilsSatz;
		this.maxSatz = maxSatz;
		this.anzahl = anzahl;
	}

	/**
	 * Konstruktor ohne Parameter für {@link JAXB}
	 * 
	 */
	public GebuehrenErhoehungsTatbestand() {
		super();
	}

	/**
	 * Ein vereinfachter Konstruktor, der die {@link #anzahl} bei
	 * Initialisierung auf -1 setzt, damit sie nachträglich noch gesetzt werden
	 * kann.
	 * 
	 * @param bezeichnung
	 *            Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *            Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param gebuehrenKlasse
	 *            Klasse der Gebühr, also den Verweis auf die Klasse eines
	 *            Ablegers von GebuehrenTabelle, also entweder
	 *            GerichtsGebuehrenTabelle.class oder
	 *            AnwaltsGebuehrenTabelle.class
	 * @param jeweilsSatz
	 *            der Satz, der für jeden der in {@link #anzahl} genannten
	 *            Personen jeweils gilt
	 * @param maxSatz
	 *            der Satz, der maximal anzusetzen ist
	 */
	GebuehrenErhoehungsTatbestand(String bezeichnung,
			Class<? extends GebuehrenTabelle> gebuehrenKlasse,
			double jeweilsSatz, double maxSatz) {
		this(bezeichnung, gebuehrenKlasse, jeweilsSatz, maxSatz, -1);
	}

	/**
	 * @return gibt {@link #anzahl} als int zurück.
	 */
	@XmlAttribute(name = "anzahl")
	public int getAnzahl() {
		return anzahl;
	}

	/**
	 * @param anzahl
	 *            d. {@link #anzahl}, d. gesetzt werden soll als int
	 */
	public void setAnzahl(int anzahl) {
		this.anzahl = anzahl;
	}

	/**
	 * @return gibt {@link #jeweilsSatz} als double zurück.
	 */
	@XmlAttribute(name = "je")
	public double getJeweilsSatz() {
		return jeweilsSatz;
	}

	/**
	 * @param jeweilsSatz d. {@link #jeweilsSatz}, d. gesetzt werden soll als double
	 */
	public void setJeweilsSatz(double jeweilsSatz) {
		this.jeweilsSatz = jeweilsSatz;
	}

	/**
	 * @return gibt {@link #maxSatz} als double zurück.
	 */
	@XmlAttribute(name = "max")
	public double getMaxSatz() {
		return maxSatz;
	}

	/**
	 * @param maxSatz d. {@link #maxSatz}, d. gesetzt werden soll als double
	 */
	public void setMaxSatz(double maxSatz) {
		this.maxSatz = maxSatz;
	}

	/**
	 * Die Methode beherbergt die "innere Logik" der GebührenErhöhung.
	 * 
	 * @return grundsätzlich {@link #jeweilsSatz} * {@link #anzahl}; wenn
	 *         {@link #maxSatz} erreicht oder überschritten wird, dieser; wenn
	 *         {@link #anzahl} noch nicht gesetzt wurde, dann wird 0.0
	 *         zurückgegeben.
	 * 
	 * @see eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand#getSatz()
	 */
	@Override
	public double getSatz() {
		if (getAnzahl() < 0)
			return 0.0;
		double satz = (double) getAnzahl() * getJeweilsSatz();
		if (satz < getMaxSatz())
			return satz;
		else
			return getMaxSatz();
	}

	/**
	 * Die toString() Methode der Klasse GebuehrenErhoehungsTatbestand liefert
	 * einen String mit der Bezeichnung des Gebührentatbestand und dem Satz in
	 * Klammern.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand#toString()
	 */
	@Override
	public String toString() {
		if (getAnzahl() < 0)
			return String
					.format("%s (%,.2f für eine bisher unbestimmte Anzahl von Personen, maximal %,.2f )",
							getBezeichnung(), getJeweilsSatz(), getMaxSatz());
		else
			return String.format("%s (%,.2f für %d Personen, maximal %,.2f )",
					getBezeichnung(), getJeweilsSatz(), getAnzahl(),
					getMaxSatz());
	}
}
