/**
 * LizenzAkzeptanzHinzufuegen.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.GridBagConstraints;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.result.FontUndFensterHelfer;

/**
 * Ein {@link HinzufuegenDialog}, der beim ersten Programmstart die Annahme der
 * Lizenzbedingungen abwartet.
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 03.12.2014
 *
 */
public class LizenzAkzeptanzHinzufuegen extends HinzufuegenDialog<Boolean> {

	private static final String KURZ_LIZENZ_TIPP = "Zeigt die für das Programm spezifische Kurzfassung der Lizenz an.";
	private static final String KURZ_LIZENZ_TEXT = "Lizenz anzeigen";
	private static final String GPL_GESAMT_TIPP = "Zeigt die gesamte Lizenz (GPL 3.0) an";
	private static final String GPL_GESAMT_TEXT = "Gesamte GPL anzeigen";

	private static final long serialVersionUID = -774138389489802048L;

	private static final String TITLE = "Lizenzvereinbarung";

	/* Für das Speichern in der Registry oder den Preferences nötige Felder */
	private static final String SCHLUESSEL = "lizenz";
	private static final String STANDARD_WERT = "";
	private static Preferences userPrefs = Preferences.userRoot().node("/eu/gronos/kostentenor");

	private JTextPane textPane;

	private HTMLEditorKit html;

	private StyledDocument doc;

	/**
	 * Je nach Betätigung dieser Action zeigt das Fenster die gesamte Lizenz (GPL
	 * 3.0) oder die für das Programm spezifische Kurzfassung der Lizenz an."
	 */
	private final Action gplAnzeigenAktion = new BeschriebeneAktion(GPL_GESAMT_TEXT, GPL_GESAMT_TIPP, KeyEvent.VK_G) {

		private static final long serialVersionUID = -5478738058066499706L;

		private boolean ausfuehrlich = false;

		@Override
		public void actionPerformed(ActionEvent e) {
			if (!ausfuehrlich) {
				htmlLesen("resources/gpl3.html");
				putValue(Action.NAME, KURZ_LIZENZ_TEXT);
				putValue(Action.SHORT_DESCRIPTION, KURZ_LIZENZ_TIPP);
				putValue(Action.MNEMONIC_KEY, KeyEvent.VK_L);
				ausfuehrlich = true;
			} else {
				htmlLesen("resources/lizenz.html");
				putValue(Action.NAME, GPL_GESAMT_TEXT);
				putValue(Action.SHORT_DESCRIPTION, GPL_GESAMT_TIPP);
				putValue(Action.MNEMONIC_KEY, KeyEvent.VK_G);
				ausfuehrlich = false;
			}
		}
	};

	/**
	 * Konstruktor: erzeugt eine Instanz der {@link LizenzAkzeptanzHinzufuegen}.
	 * 
	 * @param owner ein {@link JFrame}
	 * @throws HeadlessException
	 */
	public LizenzAkzeptanzHinzufuegen(JFrame owner) throws HeadlessException {
		this(owner, true);
	}

	/**
	 * Konstruktor: erzeugt eine Instanz der {@link LizenzAkzeptanzHinzufuegen}.
	 * 
	 * @param owner            ein {@link JFrame}
	 * @param zustimmungNoetig wenn <code>true</code>, dann wird der OK-Button in
	 *                         "Ja, ich stimme zu!" umbenannt.
	 * @throws HeadlessException
	 */
	public LizenzAkzeptanzHinzufuegen(JFrame owner, boolean zustimmungNoetig) throws HeadlessException {
		super(owner, TITLE);
		if (zustimmungNoetig) {
			AbstractAction ok = super.getOkAction();
			ok.putValue(Action.NAME, "Ja, ich stimme zu!");
			ok.putValue(Action.MNEMONIC_KEY, KeyEvent.VK_J);
			ok.putValue(Action.SHORT_DESCRIPTION, "Den Lizenzbedingungen zustimmen");
		}
		super.setOkButtonToBeFocussed(true);
	}

	/**
	 * Schreibt in den Java-Zweig der Registry (bzw. der Preferences), und zwar in
	 * den Schlüssel {@link #SCHLUESSEL}, in welcher Version die Lizenzbedingungen
	 * angenommen wurden.
	 * 
	 * @param wert <code>true</code>, wenn zugestimmt wurde, dann wird die aktuelle
	 *             Version in die Registry geschrieben. Bei <code>false</code> wird
	 *             {@link #STANDARD_WERT} in die Registry geschrieben.
	 * 
	 * @url {@link "http://openbook.galileocomputing.de/javainsel/javainsel_11_009.html#dodtp1f33b78d-493f-43a5-be9c-afc95d5f903b"}
	 *      Java ist auch eine Insel Kapitel 11.9.2 Einträge einfügen, auslesen und
	 *      löschen
	 */
	public static void inRegistrySchreiben(boolean wert) {
		if (wert)
			userPrefs.put(SCHLUESSEL, Kostenrechner.KOSTENRECHNER_VERSION);
		else
			userPrefs.put(SCHLUESSEL, STANDARD_WERT);
	}

	/**
	 * Liest aus dem Java-Zweig der Registry, unter {@link #SCHLUESSEL}, ob für die
	 * aktuelle Version des Programms die Lizenz schon akzeptiert wurde.
	 * 
	 * @return <code>true</code>, wenn in der Registry die aktuelle Version steht,
	 *         sonst <code>false</code>.
	 * @url {@link "http://openbook.galileocomputing.de/javainsel/javainsel_11_009.html#dodtp1f33b78d-493f-43a5-be9c-afc95d5f903b"}
	 *      Java ist auch eine Insel Kapitel 11.9.2 Einträge einfügen, auslesen und
	 *      löschen
	 */
	public static boolean ausRegistryLesen() {
		return Kostenrechner.KOSTENRECHNER_VERSION.equals(userPrefs.get(SCHLUESSEL, STANDARD_WERT));
	}

	/**
	 * @return {@link Boolean#TRUE} - wenn Ok gedrückt wird, ist alles gut.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Boolean baueRueckgabewert() {
		return Boolean.TRUE;
	}

	@Override
	protected void fuelleContentPane() {
		textPane = new JTextPane();
		html = new HTMLEditorKit();
		textPane.setEditorKit(html);
		textPane.setEditable(false);
		doc = (StyledDocument) html.createDefaultDocument();
		textPane.setStyledDocument(doc);
		htmlLesen("resources/lizenz.html");
		FontUndFensterHelfer fufHelfer = new FontUndFensterHelfer(this);
		fufHelfer.setPreferredSizeFromAttributeSet(doc.getParagraphElement(0).getAttributes(),
				FontUndFensterHelfer.MAX_SPALTEN, FontUndFensterHelfer.MAX_ZEILEN);
		JScrollPane scpLizenz = new JScrollPane(textPane);
		textPane.addHyperlinkListener(HinzufuegenDialog.HYPERLINK_LISTENER);
		contentPanel.add(scpLizenz, GitterBeutelBeschraenkungen.getInstance(0, 0, 3, 1, GridBagConstraints.BOTH, true));
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		super.getButtonPane().add(new JButton(gplAnzeigenAktion));
	}

	/**
	 * Die Methode dient dazu, eine HTML-Datei aus den Resources einzulesen.
	 * 
	 * @param name den Namen der HTML-Datei in den
	 *             {@link ClassLoader#getResourceAsStream(String) Resources}.
	 */
	private void htmlLesen(String name) {
		InputStream is;
		try {
			is = getClass().getClassLoader().getResourceAsStream(name);
			doc.remove(0, doc.getLength());
			doc.putProperty("IgnoreCharsetDirective", new Boolean(true));
			html.read(is, doc, 0);
		} catch (FileNotFoundException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		} catch (IOException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		} catch (BadLocationException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
		// Caret ganz nach oben setzen
		textPane.setCaretPosition(0);
	}

	/**
	 * Wenn der {@link HinzufuegenDialog#getOkAction() Ok-Button} gedrückt wird, ist
	 * alles gut.
	 * 
	 * @return <code>true</code>
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	protected boolean parseEingabe() {
		return true;
	}
}
